var JRTCSDK = require("../../JRTCSDK-Wechat.js");

const {
  JRTCClient, 
  JRTCClientInitParam, 
  JRTCClientLoginParam,
  JRTCClientCallback,
  ClientState,
  JRTCMediaDevice,
  RenderType,
  JRTCMediaDeviceVideoCanvas,
  JRTCRoom, 
  JRTCRoomInitParam, 
  JRTCRoomJoinParam,
  JRTCCallCenterCallParam,
  JRTCRecordRemoteParam,
  StompSocketAuthType,
  RoomState,
  JRTCRoomParticipant,
  RemoteRecordState,
  JRTCVersion,
  JRTCTracking,
  StorageManager,
  IntelligentMergeMode,
  VideoMergeMode,
  ScsMergeMode,
  JRTCLog,
  VideoDefinition,
  NetStatus,
  MediaType,
  JRTCMediaDeviceInitParam,
  GuestCallStateChangeType,
  JRTCGuest,
  JRTCCallCenterInitParam
} = JRTCSDK;
Page({
  context: {
    client: undefined,
    mediaDevice: undefined,
    room: undefined,
    guest: undefined,

    webSocket: undefined,
    connected: false,
    loseHeartCount: 0,
    reconnectingCount: 0,
    didClose: false,
    reconnectExceed: false,
    connectError: false,

    HEARTBEAT_TIME: 10000, // 心跳间隔，单位ms
    HEARTBEAT_MAX_COUNT: 3, // 最大心跳丢失次数
    RECONNECT_MAX_COUNT: 6, // 最大重连次数
    keepAliveIntervalTimer: undefined,//定时器ID

    streamUrlToUserIdMap: undefined,
    localCanvas: undefined,
    remoteCanvas: undefined,

    mergeParticipant: undefined,//合流成员
    testCount: 0, // 初始化次数计数

    networkType: undefined, //当前网络类型
  },

  /**
   * 页面的初始数据
   */
  data: {
    deviceName: "juphoon_WeChat",
    version: '',
    webSocketUrl: "ws://192.168.2.50:10050/ws",

    remoteVideoShow: false,
    localVideoShow: false,

    logContent: [],
    scrollTo: "",
    endId: "",

    toastInfo: {
      textToast: false,
      hideTextToast: false,
      contentToast: '',
    },
  },

  /**
   * 生命周期函数--监听页面加载
   */
  onLoad(options) {
    //初始化缓存数据
    this.data.version = JRTCVersion.getSDKVersion();
    let deviceName = wx.getStorageSync('deviceName');
    let webSocketUrl = wx.getStorageSync('webSocketUrl');
    if (this.isEmpty(deviceName)) {
      deviceName = this.data.deviceName;
    } 
    if (this.isEmpty(webSocketUrl)) {
      webSocketUrl = this.data.webSocketUrl;
    } 
    this.setData({deviceName: deviceName, webSocketUrl: webSocketUrl});

    let that = this;
    wx.getNetworkType({
      success (res) {
        that.context.networkType = res.networkType;
      }
    });
    wx.onNetworkStatusChange(function (res) {
      that.updateLog('onNetworkStatusChange:' + res.networkType + " " + res.isConnected);
      that.context.networkType = res.networkType;
      if (res.isConnected && res.networkType == 'wifi') {
        if ((that.context.reconnectExceed || that.context.connectError)) {
          if (that.context.webSocket) {
            if (!that.context.connected) {
              that.disConnectWebSocket('4002', '网络切换到wifi断开正在连接socket');
            }
          } else {
            that.reconnect();
          }
        }
      }
    })
  },

  /**
   * 生命周期函数--监听页面初次渲染完成
   */
  onReady() {
    
  },

  /**
   * 生命周期函数--监听页面显示
   */
  onShow() {

  },

  /**
   * 生命周期函数--监听页面隐藏
   */
  onHide() {

  },

  /**
   * 生命周期函数--监听页面卸载
   */
  onUnload() {
    wx.setStorageSync('deviceName', this.data.deviceName);
    wx.setStorageSync('webSocketUrl', this.data.webSocketUrl);
  },

  /**
   * 页面相关事件处理函数--监听用户下拉动作
   */
  onPullDownRefresh() {

  },

  /**
   * 页面上拉触底事件的处理函数
   */
  onReachBottom() {

  },

  /**
   * 用户点击右上角分享
   */
  onShareAppMessage() {

  },

  bindInput: function (ev) {
    let _a;
    let path = ev.currentTarget.dataset['path'];
    this.setData((_a = {},
      _a[path] = ev.detail.value,
      _a));
  },

  connectWebSocket() {
    if (this.notUAN(this.context.webSocket)) {
        this.updateLog('已经存在连接...');
        return;
    }
    if (this.isEmpty(this.data.webSocketUrl)) { 
        this.openTextToast('自动化测试服务地址不能为空');
        return;
    }
    if (this.isEmpty(this.data.deviceName)) {
        this.openTextToast('设备名称不能为空');
        return;
    }
    let that = this;
    this.context.webSocket = wx.connectSocket({
      url: this.data.webSocketUrl,
      header:{
        'content-type': 'application/json'
      },    
      success: function(res) {
          console.info("connectSocket success ", res);
          that.updateLog("连接 socket 接口调用成功");
      },
      fail: function(res) {
          that.context.webSocket = undefined;
          console.error("connectSocket fail ", res);
          that.updateLog("连接 socket 接口调用失败");
      }
    });
    this.context.webSocket.onOpen(res => {
      that.context.connectError = false;
      that.context.connected = true;
      that.context.reconnectingCount = 0;
      //连接成功...
      that.updateLog("连接成功...", false);
      that.sendMessage(that.createDeviceAddJsonString());
      that.startKeepAlive();
    });
    this.context.webSocket.onClose(res => {
      that.updateLog("连接关闭..." + " code -> " + res.code + "原因 -> " + res.reason);
      that.context.webSocket = undefined;
      that.context.connected = false;
      that.stopKeepAlive();
      if (res.code == 4000) { //主动断开
          that.context.reconnectingCount = 0;
          that.context.didClose = false;
      } else {
          that.reconnect();
      }
    });
    this.context.webSocket.onError(res => {
        //console.error("WebSocket error observed:", res);
        that.updateLog("连接错误... " + res.errMsg);
        that.context.connectError = true;
        that.disConnectWebSocket(4002, '连接错误断开');
    });
 
    this.context.webSocket.onMessage(res => {
        //收到消息...
        that.updateLog("收到消息：\n" + res.data);
        that.handleReceivedMessage(res.data);
    });
  },

  reconnect() {
    this.context.reconnectingCount ++;
    if (this.context.reconnectingCount > this.context.RECONNECT_MAX_COUNT) {
        this.updateLog("重连超过最大" + this.context.RECONNECT_MAX_COUNT + "次，结束重连");
        this.context.reconnectExceed = true;
        this.context.reconnectingCount = 0;
    } else {
        this.context.reconnectExceed = false;
        this.updateLog("开始第" + this.context.reconnectingCount + "次重连");
        if (!this.context.didClose) {//如果主动断开连接就不用再重连了
            this.connectWebSocket();
        } else {
            this.context.didClose = false;
            this.updateLog("重连终止... 原因 -> 主动断开连接");
        }
    }
  },

  tapDisConnectWebSocket() {
    this.disConnectWebSocket(4000, "主动断开");
  },

  disConnectWebSocket(code, reason) {
    this.updateLog(reason);
    this.stopKeepAlive();
    if (this.context.webSocket) {
        this.sendMessage(this.createDeviceRemoveJsonString());
        //延时关闭保证移除设备消息发送成功
        let that = this;
        setTimeout(() => {
            if (code == 4000) {
              that.context.reconnectExceed = false;
              that.context.didClose = true;
            }
            if (that.context.webSocket) {
                that.context.webSocket.close({
                  code: code, 
                  reason: reason,
                  success: function () {
                    that.updateLog("关闭 websocket 成功");
                  },
                  fail: function () {
                    that.updateLog("关闭 websocket 失败");
                  },
                  complete: function () {
                    that.updateLog("关闭 websocket 完成");
                    if (code == 4002) {
                      if (that.context.networkType == 'wifi') {
                        that.reconnect();
                      }
                    }
                  }
                });
                that.context.webSocket = undefined;
            }
        }, 500);
    } else {
        if (code == 4001) {//重连
            this.reconnect();
        } else {
            this.updateLog("未存在连接...");
        }
    }
  },

  startKeepAlive() {
    if (!this.context.keepAliveIntervalTimer) {
        this.updateLog("开始心跳");
        this.context.loseHeartCount = 0;
        this.context.keepAliveIntervalTimer = setInterval(this.sendHeartbeat, 10000);
        this.sendHeartbeat();
    }
  },

  sendHeartbeat() {
    this.context.loseHeartCount ++;
    if (this.context.loseHeartCount > this.context.HEARTBEAT_MAX_COUNT) {
        this.disConnectWebSocket(4001, "心跳超时断开");
    } else {
        this.updateLog("发送心跳包");
        this.sendMessage(JSON.stringify({"type": "request", "name": "heartbeat"}));
    }
  },

  stopKeepAlive() {
      this.context.loseHeartCount = 0;
      if (this.context.keepAliveIntervalTimer) {
          this.updateLog("停止心跳");
          clearInterval(this.context.keepAliveIntervalTimer);
          this.context.keepAliveIntervalTimer = undefined;
      }
  },

  dealKeepAliveMessage(data) {
      if (data && data.result && data.result.boolValue === true) {
          //心跳成功
          this.context.loseHeartCount = 0;
          let showText = data.description;
          if (this.isEmpty(showText)) {
              showText = "心跳成功";
          }
          this.updateLog(showText);
      }
  },

  updateLog(text) {
    console.log(text);
    if (this.isNotEmpty(text)) {
      if (this.data.logContent.length >= 50) {
         this.data.logContent.splice(0, 1);
      }
      this.data.logContent.push(text);
      this.setData({
        logContent: this.data.logContent
      });
      this.autoScroll();
    }
  },

  autoScroll() {
    let view_id = 'view_id_' + parseInt(Math.random() * 1000000)
    this.setData({
      scrollTo: ''
    })
    this.setData({
      endId: view_id
    })
    this.setData({
      scrollTo: view_id
    })
  },

  clearLog() {
    this.setData({
      logContent: []
    });
  },

  sendMessage(text) {
    if (this.context.webSocket && this.context.connected) {
      let that = this;
      this.context.webSocket.send({
        data: text,
        success: function () {
          that.updateLog("发送消息成功");
        },
        fail: function () {
          that.updateLog("发送消息失败");
        },
        complete: function () {
          that.updateLog("发送消息完成");
        }
      });
      this.updateLog("发送消息：\n" + text);
    }
  },

  handleReceivedMessage(text) {
    let data = JSON.parse(text);
    let name = data.name;
    let type = data.type;
    console.log("handleReceivedMessage: ", text);
    if ("request" === type) {
      let description = data.description;
      let dm = data.dm;
      let parameters = data.parameters;
      let returnValue = undefined;
      let async = false;
      switch (dm) {
        case "JRTCClient":
          switch (name) {
            case "create":
              // 清除console缓存
              if (this.context.testCount > 100) {
                this.context.testCount = 0;
                this.clearLog();
                console.clear();
              }
              this.context.testCount ++;
            
              const clientInitParam = new JRTCClientInitParam();
              clientInitParam.appName = parameters.initParam.appName;
              clientInitParam.server = parameters.initParam.server;
              clientInitParam.appKey = parameters.initParam.appKey;
              clientInitParam.logConsole = parameters.initParam.logConsole;
              this.context.client = JRTCClient.create({
                onLogin: (result, reason) => {
                    this.sendMessage(this.createResponseJsonString("onLogin", "登录结果回调", 
                    dm, {result: result, reason: reason}));
                },
                onLogout: (reason) => {
                    this.sendMessage(this.createResponseJsonString("onLogout", "登出结果回调", 
                    dm, {reason: reason}));
                },
                onClientStateChanged: (state, oldState) => {
                    this.sendMessage(this.createResponseJsonString("onClientStateChanged", "登录状态变化通知", 
                    dm, {state: state, oldState: oldState}));
                },
                onSDKEvent: (event) => {
                    this.sendMessage(this.createResponseJsonString("onSDKEvent", "上报事件回调", 
                    dm, {event: event}));
                },
                onOnlineMessageReceived: (message, userId) => {
                    this.sendMessage(this.createResponseJsonString("onOnlineMessageReceived", "收到在线消息回调", 
                    dm, {message: message, userId: userId}));
                },
                onOnlineMessageSendResult: (result, operatorId) => {
                    this.sendMessage(this.createResponseJsonString("onOnlineMessageSendResult", "在线消息发送结果回调", 
                    dm, {result: result, operatorId: operatorId}));
                }
              }, clientInitParam);
              description = "创建 JRTCClient 对象" + (this.notUAN(this.context.client) ? "成功" : "失败" );
              returnValue = this.notUAN(this.context.client);
              break;
            case "destroy":
              if (this.notUAN(this.context.client)) {
                JRTCClient.destroy();
                this.context.client = undefined;
              }
              returnValue = true;
              break;
            case "getUserId":
              returnValue = this.context.client.getUserId();
              break;
            case "getState":
              returnValue = this.context.client.getState();
              break;
            case "getDisplayName":
              returnValue = this.context.client.getDisplayName();
              break;
            case "setDisplayName":
              this.context.client.setDisplayName(parameters.displayName);
              returnValue = true;
            case "getServer":
              returnValue = this.context.client.getServer();
              break;
            case "setServer":
              this.context.client.setServer(parameters.server);
              returnValue = true;
              break;
            case "getAppKey":
              returnValue = this.context.client.getAppKey();
              break;
            case "setAppKey":
              this.context.client.setAppKey(parameters.appKey);
              returnValue = true;
              break;
            case "getAppName":
              returnValue = this.context.client.getAppName();
              break;
            case "setAppName":
              this.context.client.setAppName(parameters.appName);
              returnValue = true;
              break;
            case "login":
              let loginParam = new JRTCClientLoginParam();
              if (parameters.clientLoginParam) {
                  loginParam.token = parameters.clientLoginParam.token;
                  loginParam.tokenType = parameters.clientLoginParam.tokenType;
              }
              returnValue = this.context.client.login(parameters.userId, parameters.password, loginParam);
              break;
            case "logout":
              returnValue = this.context.client.logout();
              break;
            case "sendOnlineMessage":
              returnValue = this.context.client.sendOnlineMessage(parameters.message, parameters.userId);
              break;
            default:
              description = "未找到该方法";
              returnValue = false;
              break;
          }
          break;
        case "JRTCMediaDevice":
          switch (name) {
              case "create":
              let mediaDeviceInitParam = new JRTCMediaDeviceInitParam();
              this.context.mediaDevice = JRTCMediaDevice.create({
                  onCameraUpdate: ()=> {
                      this.sendMessage(this.createResponseJsonString("onCameraUpdate", "摄像头变化回调", 
                      dm, {result: true}));
                  },
                  onAudioInputUpdate: ()=> {
                      this.sendMessage(this.createResponseJsonString("onAudioInputUpdate", "输入音频变化回调", 
                      dm, {result: true}));
                  }
              }, mediaDeviceInitParam);
              returnValue = this.notUAN(this.context.mediaDevice);
              break;
            case "destroy":
              if (this.notUAN(this.context.mediaDevice)) {
                this.context.mediaDevice.stopAllMedia();
                this.context.localCanvas = undefined;
                this.context.remoteCanvas = undefined;
                this.removeAllVideo();
                JRTCMediaDevice.destroy();
                this.context.mediaDevice = undefined;
              }
              returnValue = true;
              break;
            case "setCameraProperty":
              this.context.mediaDevice.setCameraProperty(parameters.width, parameters.height, parameters.frameRate);
              returnValue = true;
              break;
            case "startCameraVideo": {
              let viewId = 'local-stream';
              this.showLocalVideo();
              this.context.mediaDevice.startCameraVideo(parameters.renderType, viewId, this, parameters.enableMic).then((canvas) => {
                let returnValue;
                if (this.notUAN(canvas)) {
                  this.context.localCanvas = canvas;
                  returnValue = canvas.streamUrl;
                } else {
                  returnValue = undefined;
                }
                this.sendMessage(this.createReturnJsonString(name, description, dm, returnValue));
              }).catch((e) => {
                console.error('本地视频渲染失败', e);
                this.openTextToast('本地视频渲染失败');
                this.sendMessage(this.createReturnJsonString(name, description, dm, undefined));
              });
              async = true;
              break;
            }
            case "startVideo": {
              let viewId = 'remote-stream';
              this.showRemoteVideo();
              this.context.remoteCanvas = this.context.mediaDevice.startVideo(parameters.streamUrl, parameters.renderType, viewId, this);
              if (this.notUAN(this.context.remoteCanvas)) {
                returnValue = this.context.remoteCanvas.streamUrl;
              } else {
                this.openTextToast('远端视频渲染视频失败');
                returnValue = undefined;
              }
              break;
            }
            case "stopVideo": {
              let streamUrl = parameters.streamUrl;
              if (streamUrl === this.context.localCanvas.streamUrl) {
                this.context.mediaDevice.stopVideo(this.context.localCanvas);
                this.context.localCanvas = undefined;
                this.removeLocalVideo();
              } else if (streamUrl === this.context.remoteCanvas.streamUrl) {
                this.context.mediaDevice.stopVideo(this.context.remoteCanvas);
                this.context.remoteCanvas = undefined;
                this.removeRemoteVideo();
              }
              returnValue = true;
              break;
            }
            case "stopAllMedia":
              this.context.mediaDevice.stopAllMedia();
              this.context.localCanvas = undefined;
              this.context.remoteCanvas = undefined;
              this.removeAllVideo();
              returnValue = true;
              break;
            case "startAudioOutput":
              returnValue = this.context.mediaDevice.startAudioOutput();
              break;
            case "stopAudioOutput":
              returnValue = this.context.mediaDevice.stopAudioOutput();
              break;
            case "switchCamera":
              returnValue = this.context.mediaDevice.switchCamera();
              break;
            case "enableSpeaker":
              this.context.mediaDevice.enableSpeaker(parameters.enable);
              returnValue = true;
              break;
            case "isSpeakerOn":
              returnValue = this.context.mediaDevice.isSpeakerOn();
              break;
            case "isCameraOpen":
              returnValue = this.context.mediaDevice.isCameraOpen();
              break;
            case "getCurrentCamera":
              returnValue = this.context.mediaDevice.getCurrentCamera();
              break;
            case "getCameras":
              returnValue = this.context.mediaDevice.getCameras();
              break;
            case "isAudioStart":
              returnValue = this.context.mediaDevice.isAudioStart();
              break;
            default:
              description = "未找到该方法";
              returnValue = false;
              break;
          }
          break;
        case "JRTCRoom":
          switch (name) {
            case "create":
              this.context.streamUrlToUserIdMap = new Map();
              const roomInitParam = new JRTCRoomInitParam();
              roomInitParam.roomServer = parameters.initParam.roomServer;
              roomInitParam.protocol = parameters.initParam.protocol;
              this.context.room = JRTCRoom.create(this.context.client, this.context.mediaDevice, roomInitParam, {
                  onRoomStateChanged: (state, oldState) => {
                      this.sendMessage(this.createResponseJsonString("onRoomStateChanged", "自身在房间中的状态变化回调", 
                      dm, {state: state, oldState: oldState}));
                  },
                  onRoomPropertyChanged: (changeParam) => {
                      this.sendMessage(this.createResponseJsonString("onRoomPropertyChanged", "房间属性变化回调", 
                      dm, {changeParam: changeParam}));
                  },
                  onJoin: (result, reason, roomId) => {
                      this.context.streamUrlToUserIdMap.clear();
                      this.sendMessage(this.createResponseJsonString("onJoin", "加入房间结果回调", 
                      dm, {result: result, reason: reason, roomId: roomId}));
                  },
                  onLeave: (reason, roomId) => {
                      this.context.streamUrlToUserIdMap.clear();
                      this.sendMessage(this.createResponseJsonString("onLeave", "离开房间结果回调", 
                      dm, {reason: reason, roomId: roomId}));
                  },
                  onQuery: (operationId, result, reason, queryInfo) => {
                      this.context.sendMessage(this.createResponseJsonString("onQuery", "查询房间信息结果回调", 
                      dm, {operationId: operationId, result: result, reason: reason, queryInfo: queryInfo}));
                  },
                  onParticipantJoin: (participant) => {
                      if ("MERGED__" == participant.userId) {
                          this.mergeParticipant = participant;//保存合流成员
                      }
                      this.sendMessage(this.createResponseJsonString("onParticipantJoin", "新成员加入回调", 
                      dm, {participant: participant}));
                  },
                  onParticipantLeft: (participant, reason) => {
                      if ("MERGED__" == participant.userId) {
                          this.context.mergeParticipant = undefined;
                      }
                      this.sendMessage(this.createResponseJsonString("onParticipantLeft", "成员离开回调", 
                      dm, {participant: participant, reason: reason}));
                  },
                  onParticipantUpdate: (participant, changeParam) => {
                      this.sendMessage(this.createResponseJsonString("onParticipantUpdate", "成员更新回调", 
                      dm, {participant: participant, changeParam: changeParam}));
                  },
                  onMessageReceived: (type, content, fromUserId) => {
                      this.sendMessage(this.createResponseJsonString("onMessageReceived", "接收房间消息的回调", 
                      dm, {type: type, content: content, fromUserId: fromUserId}));
                  },
                  onDeliveryAbort: (isShutDown, deliveryUserId, reason) => {
                      this.sendMessage(this.createResponseJsonString("onDeliveryAbort", "录制异常回调", 
                      dm, {isShutDown: isShutDown, deliveryUserId: deliveryUserId, reason: reason}));
                  }
              });
              returnValue = this.notUAN(this.context.room);
              break;
            case "destroy":
              if (this.notUAN(this.context.room)) {
                this.context.streamUrlToUserIdMap = undefined;
                JRTCRoom.destroy();
                this.context.room = undefined;
              }
              returnValue = true;
              break;
            case "enableUploadAudioStream":
                returnValue = this.context.room.enableUploadAudioStream(parameters.enable);
                break;
            case "enableUploadVideoStream":
                returnValue = this.context.room.enableUploadVideoStream(parameters.enable);
                break;
            case "getRoomId":
                returnValue = this.context.room.getRoomId();
                break;
            case "getPassword":
                returnValue = this.context.room.getPassword();
                break;
            case "getCallId":
                returnValue = this.context.room.getCallId();
                break;
            case "getTitle":
                returnValue = this.context.room.getTitle();
                break;
            case "getState":
                returnValue = this.context.room.getState();
                break;
            case "getParticipants":
                returnValue = this.context.room.getParticipants();
                break;
            case "getParticipant":
                returnValue = this.context.room.getParticipant(parameters.userId);
                break;
            case "statistics":
                returnValue = this.context.room.statistics();
                break;
            case "getUploadLocalAudio":
                returnValue = this.context.room.getUploadLocalAudio();
                break;
            case "getUploadLocalVideo":
                returnValue = this.context.room.getUploadLocalVideo();
                break;
            case "getShareStreamId":
                returnValue = this.context.room.getShareStreamId();
                break;
            case "getShareUserId":
                returnValue = this.context.room.getShareUserId();
                break;
            case "getRemoteRecordState":
                returnValue = this.context.room.getRemoteRecordState();
                break;
            case "join":
                let joinParam = new JRTCRoomJoinParam();
                Object.keys(parameters.joinParam).forEach(key => {
                    joinParam[key] = parameters.joinParam[key];
                });
                returnValue = this.context.room.join(parameters.roomId, joinParam);
                break;
            case "leave":
                returnValue = this.context.room.leave();
                break;
            case "requestVideo":
              let participant = undefined;
              if (parameters.userId === 'MERGED__') { //合流成员
                  participant = this.context.mergeParticipant;
              } else {
                  participant = this.context.room.getParticipant(parameters.userId);
              }
              this.context.room.requestVideo(participant, parameters.videoSize).then((streamUrl) => {
                  let returnValue = streamUrl;
                  if (this.notUAN(streamUrl)) {
                    this.context.streamUrlToUserIdMap.set(streamUrl, participant.userId);
                  }
                  this.sendMessage(this.createReturnJsonString(name, description, dm, returnValue));
              }).catch(err => {
                  console.error("requestVideo ", err);
                  this.sendMessage(this.createReturnJsonString(name, description, dm, undefined));
              });
              async = true;
              break;
            case "unRequestVideo":
              returnValue = this.context.room.unRequestVideo(this.context.room.getParticipant(parameters.userId));
              break;
            case "enableRemoteRecord":
                returnValue = this.context.room.enableRemoteRecord(parameters.enable, parameters.recordParam);
                break;
            case "sendMessage":
                returnValue = this.context.room.sendMessage(parameters.type, parameters.content, parameters.toUserId);
                break;
            case "setRatio":
                returnValue = this.context.room.setRatio(parameters.ratio);
                break;
            case "isAutoRecord":
                returnValue = this.context.room.isAutoRecord();
                break;
            default:
              description = "未找到该方法";
              returnValue = false;
              break;
          }
          break;
          case "JRTCGuest":
            switch (name) {
              case "create":
                this.context.streamUrlToUserIdMap = new Map();
                const guestInitParam = new JRTCCallCenterInitParam();
                guestInitParam.roomServer = parameters.initParam.roomServer;
                guestInitParam.protocol = parameters.initParam.protocol;
                this.context.guest = JRTCGuest.create(this.context.client, this.context.mediaDevice, guestInitParam, {
                    onGetAllGroups: (result, groups) => {
                        this.sendMessage(this.createResponseJsonString("onGetAllGroups", "查询业务组号回调", 
                        dm, {result: result, groups: groups}));
                    },
                    onCallStateChanged: (type, incomingType, inviter, reason) => {
                        if (type === GuestCallStateChangeType.TERMED) {
                            this.context.streamUrlToUserIdMap.clear();
                        }
                        this.sendMessage(this.createResponseJsonString("onCallStateChanged", "通话状态改变", 
                        dm, {type: type, incomingType: incomingType, inviter: inviter, reason: reason}));
                    },
                    onCallQueueCount: (count, time, agentRinging) => {
                        this.sendMessage(this.createResponseJsonString("onCallQueueCount", "当前排队人数上报回调", 
                        dm, {count: count, time: time, agentRinging: agentRinging}));
                    },
                    onCallPropertyChanged: (propChangeParam) => {
                        this.sendMessage(this.createResponseJsonString("onCallPropertyChanged", "通话属性改变回调", 
                        dm, {propChangeParam: propChangeParam}));
                    },
                    onMemberJoin: (participant) => {
                        if ("MERGED__" == participant.userId) {
                            this.mergeParticipant = participant;//保存合流成员
                        }
                        this.sendMessage(this.createResponseJsonString("onMemberJoin", "通话中有新成员加入回调", 
                        dm, {participant: participant}));
                    },
                    onMemberLeave: (participant) => {
                        if ("MERGED__" == participant.userId) {
                            this.mergeParticipant = undefined;
                        }
                        this.sendMessage(this.createResponseJsonString("onMemberLeave", "通话中有新成员离开回调", 
                        dm, {participant: participant}));
                    },
                    onMemberUpdate: (participant, changeParam) => {
                        this.sendMessage(this.createResponseJsonString("onMemberUpdate", "通话中成员属性更新回调", 
                        dm, {participant: participant, changeParam: changeParam}));
                    },
                    onMessageReceived: (content, contentType, messageType, fromUserId) => {
                        this.sendMessage(this.createResponseJsonString("onMessageReceived", "收到消息回调", 
                        dm, {content: content, contentType: contentType, messageType: messageType, fromUserId: fromUserId}));
                    },
                    onSignRequest: (fromUserId, extraInfo) => {
                        this.sendMessage(this.createResponseJsonString("onSignRequest", "收到签名请求回调", 
                        dm, {fromUserId: fromUserId, extraInfo: extraInfo}));
                    },
                    onDeliveryAbort: (isShutDown, deliveryUserId, reason) => {
                        this.sendMessage(this.createResponseJsonString("onDeliveryAbort", "录制异常回调", 
                        dm, {isShutDown: isShutDown, deliveryUserId: deliveryUserId, reason: reason}));
                    },
                    onNotifyMessageReceived: (notifyMessage, fromUserId) => {
                        this.sendMessage(this.createResponseJsonString("onNotifyMessageReceived", "收到事件通知消息回调", 
                        dm, {notifyMessage: notifyMessage, fromUserId: fromUserId}));
                    },
                    onUrgentResultResponse: (agree) => {
                        this.sendMessage(this.createResponseJsonString("onUrgentResultResponse", "座席处理加急的结果回调", 
                        dm, {agree: agree}));
                    }
                });
                returnValue = this.notUAN(this.context.guest);
                break;
              case "destroy":
                  if (this.notUAN(this.context.guest)) {
                    this.context.streamUrlToUserIdMap = undefined;
                    JRTCGuest.destroy();
                    this.context.guest = undefined;
                  }
                  returnValue = true;
                  break;
              case "getCallState":
                  returnValue = this.context.guest.getCallState();
                  break;
              case "isMainGuest":
                  returnValue = this.context.guest.isMainGuest();
                  break;
              case "getMainAgentParticipant":
                  returnValue = this.context.guest.getMainAgentParticipant();
                  break;
              case "getAgentParticipants":
                  returnValue = this.context.guest.getAgentParticipants();
                  break;
              case "getSelfParticipant":
                  returnValue = this.context.guest.getSelfParticipant();
                  break;
              case "getParticipants":
                  returnValue = this.context.guest.getParticipants();
                  break;
              case "getSerialNumber":
                  returnValue = this.context.guest.getSerialNumber();
                  break;
              case "getCallId":
                  returnValue = this.context.guest.getCallId();
                  break;
              case "getShareStreamId":
                  returnValue = this.context.guest.getShareStreamId();
                  break;
              case "getShareUserId":
                  returnValue = this.context.guest.getShareUserId();
                  break;
              case "setMultiStream":
                  this.context.guest.setMultiStream(parameters.multiStream);
                  returnValue = true;
                  break;
              case "queryAllGroups":
                  returnValue = this.context.guest.queryAllGroups();
                  break;
              case "call": {
                  let callParam = new JRTCCallCenterCallParam();
                  if (parameters.callParam) {
                      Object.keys(parameters.callParam).forEach(key => {
                          callParam[key] = parameters.callParam[key];
                      });
                  }
                  returnValue = this.context.guest.call(parameters.number, callParam);
                  break;
              }
              case "oneToOneCall": {
                  let callParam = new JRTCCallCenterCallParam();
                  Object.keys(parameters.callParam).forEach(key => {
                      callParam[key] = parameters.callParam[key];
                  });
                  returnValue = this.context.guest.oneToOneCall(parameters.userId, callParam);
                  break;
              }
              case "answer":
                  returnValue = this.context.guest.answer();
                  break;
              case "term":
                  returnValue = this.context.guest.term();
                  break;
              case "requestUrgent":
                  returnValue = this.context.guest.requestUrgent();
                  break;
              case "sendMessage":
                  returnValue = this.context.guest.sendMessage(parameters.contentType, parameters.content, parameters.toUserId);
                  break;
              case "sendNotifyMessage":
                  returnValue = this.context.guest.sendNotifyMessage(parameters.notifyMessage, parameters.toUserId);
                  break;
              case "enableUploadAudioStream":
                  returnValue = this.context.guest.enableUploadAudioStream(parameters.enable);
                  break;
              case "enableUploadVideoStream":
                  returnValue = this.context.guest.enableUploadVideoStream(parameters.enable);
                  break;
              case "setRatio":
                  returnValue = this.context.guest.setRatio(parameters.ratio);
                  break;
              case "getRequestSize":
                  returnValue = this.context.guest.getRequestSize();
                  break;
              case "setRequestSize":
                  returnValue = this.context.guest.setRequestSize(parameters.requestSize);
                  break;
              case "requestVideo": {
                let participant = undefined;
                if (parameters.userId === 'MERGED__') { //合流成员
                    participant = this.context.mergeParticipant;
                } else {
                    participant = this.getParticipant(this.context.guest.getParticipants(), parameters.userId);
                }
                this.context.guest.requestVideo(participant, parameters.videoSize).then((streamUrl) => {
                    let returnValue = streamUrl;
                    if (this.notUAN(streamUrl)) {
                        this.context.streamUrlToUserIdMap.set(streamUrl, participant.userId);
                    }
                    this.sendMessage(this.createReturnJsonString(name, description, dm, returnValue));
                }).catch(err => {
                    console.error("requestVideo ", err);
                    this.sendMessage(this.createReturnJsonString(name, description, dm, undefined));
                });
                async = true;
                break;
              }
              case "unRequestVideo":
                returnValue = this.context.guest.unRequestVideo(this.getParticipant(this.context.guest.getParticipants(), parameters.userId));
                break;
              case "enableRemoteRecord":
                  returnValue = this.context.guest.enableRemoteRecord(parameters.enable, parameters.recordParam);
                  break;
              case "getRemoteRecordState":
                  returnValue = this.context.guest.getRemoteRecordState();
                  break;
              case "responseSignResult":
                  returnValue = this.context.guest.responseSignResult(parameters.result, parameters.userId, parameters.extraInfo);
                  break;
              case "statistics":
                  returnValue = this.context.guest.statistics();
                  break;
              default:
                description = "未找到该方法";
                returnValue = false;
                break;
            }
            break;
        default:
          switch(name) {
            case "stopTest":
              if (this.context.guest) {
                  this.context.streamUrlToUserIdMap = undefined;
                  JRTCGuest.destroy();
                  this.context.guest = undefined;
              }
              if (this.context.room) {
                  this.context.streamUrlToUserIdMap = undefined;
                  JRTCRoom.destroy();
                  this.context.room = undefined;
              }
              if (this.context.mediaDevice) {
                  this.context.mediaDevice.stopAllMedia();
                  this.context.localCanvas = undefined;
                  this.context.remoteCanvas = undefined;
                  this.removeAllVideo();
                  JRTCMediaDevice.destroy();
                  this.context.mediaDevice = undefined;
              }
              if (this.context.client) {
                  JRTCClient.destroy();
                  this.context.client = undefined;
              }
              returnValue = true;
              break;
            default:
              description = "未找到该方法";
              returnValue = false;
              break;
          }
          break;
      }
      if (!async) {
        this.sendMessage(this.createReturnJsonString(name, description, dm, returnValue));
      }
    } else if ("response" === type) {
      if ("heartbeat" === name) {
          this.dealKeepAliveMessage(data);
      }
    }
  },

  getParticipant(participants, userId) {
    for (const p of participants) {
        if (p.userId === userId) {
            return p;
        }
    }
    return undefined;
  },

  createReturnJsonString(name, description, dm, result) {
      let data = this.createCommonJsonObj('return', name, description);
      data.dm = dm;
      data.result = result;
      return JSON.stringify(data);
  },

  createResponseJsonString(name, description, dm, result) {
      let data = this.createCommonJsonObj('response', name, description);
      data.dm = dm;
      data.result = result;
      return JSON.stringify(data);
  },

  createCommonJsonObj(type, name, description) {
      let data = {
          type: type,
          name: name,
          description: description,
          time: new Date().getTime()
      }
      return data;
  },

  createDeviceAddJsonString() {
      let data = this.createCommonJsonObj('request', 'deviceAdd', '添加设备');
      data.system = 'Wechat';
      data.nickname = this.data.deviceName;
      data.version = this.data.version;
      data.deviceId = this.uuid();
      return JSON.stringify(data);
  },

  createDeviceRemoveJsonString() {
      let data = this.createCommonJsonObj('request', 'deviceRemove', '移除设备');
      data.system = 'H5';
      data.nickname = this.deviceName;
      return JSON.stringify(data);
  },

  removeAllVideo() {
    this.removeLocalVideo();
    this.removeRemoteVideo();
  },

  showLocalVideo() {
    this.setData({
      localVideoShow: true
    });
  },

  removeLocalVideo() {
    this.setData({
      localVideoShow: false
    });
  },

  showRemoteVideo() {
    this.setData({
      remoteVideoShow: true
    });
  },

  removeRemoteVideo() {
    this.setData({
      remoteVideoShow: false
    });
  },

  uuid() {
      let s = [];
      let hexDigits = "0123456789abcdef";
      for (let i = 0; i < 36; i++) {
          let from = Math.floor(Math.random() * 0x10);
          s[i] = hexDigits.substring(from, from + 1);
      }
      s[14] = "4"; // bits 12-15 of the time_hi_and_version field to 0010
      let from = (s[19] & 0x3) | 0x8;
      s[19] = hexDigits.substring(from, from + 1); // bits 6-7 of the clock_seq_hi_and_reserved to 01
      s[8] = s[13] = s[18] = s[23] = "-";
  
      let uuid = s.join("");
      return uuid;
  },

  notUAN(object) {
      if (object != null && object != undefined) {
          return true;
      } else {
          return false;
      }
  },

  isEmpty(str) {
    if (
      str === null ||
          str === '' ||
          str === undefined ||
          str.length === 0 || str.trim().length === 0
    ) {
      return true
    } else {
      return false
    }
  },

  isNotEmpty(str) {
      if (
          str === null ||
              str === '' ||
              str === undefined ||
              str.length === 0 || str.trim().length === 0
        ) {
          return false
        } else {
          return true
        }
  },

  openTextToast(content) {
    this.setData({
      toastInfo: {
        textToast: true,
        contentToast: content,
      }
    });
    setTimeout(() => {
      this.setData({
        toastInfo: {
          hideTextToast: true,
        }
      });
      setTimeout(() => {
        this.setData({
          toastInfo: {
            textToast: false,
            hideTextToast: false,
            contentToast: '',
          }
        });
      }, 300);
    }, 3000);
  },
});